@@ -19,10 +19,10 @@ build_time_vars = {'ABIFLAGS': 'm',
  'BINLIBDEST': '/usr/local/lib/python3.4',
  'BLDLIBRARY': '-L. -lpython3.4m',
  'BLDSHARED': 'cc -pthread -shared -fPIC -L/usr/local/lib/ '
-              '-L/usr/obj/ports/Python-3.4.2/Python-3.4.2',
+              '-L/usr/obj/ports/Python-3.4.3/Python-3.4.3',
  'BUILDEXE': '',
  'BUILDPYTHON': 'python',
- 'BUILD_GNU_TYPE': 'mips64-unknown-openbsd5.7',
+ 'BUILD_GNU_TYPE': 'sparc64-unknown-openbsd5.8',
  'BYTESTR_DEPS': '\\',
  'CC': 'cc -pthread',
  'CCSHARED': '-fPIC',
@@ -37,7 +37,7 @@ build_time_vars = {'ABIFLAGS': 'm',
  'CONFIGURE_CPPFLAGS': '-I/usr/local/include',
  'CONFIGURE_LDFLAGS': '-L/usr/local/lib/',
  'CONFIG_ARGS': "'--with-ensurepip=no' '--enable-shared' "
-                "'--srcdir=/usr/obj/ports/Python-3.4.2/Python-3.4.2' "
+                "'--srcdir=/usr/obj/ports/Python-3.4.3/Python-3.4.3' "
                 "'--with-fpectl' '--with-threads' '--enable-ipv6' "
                 "'--with-cxx_main' '--with-system-expat' '--with-system-ffi' "
                 "'--prefix=/usr/local' '--sysconfdir=/etc' "
@@ -48,8 +48,8 @@ build_time_vars = {'ABIFLAGS': 'm',
  'CONFINCLUDEDIR': '/usr/local/include',
  'CONFINCLUDEPY': '/usr/local/include/python3.4m',
  'COREPYTHONPATH': ':plat-openbsd5',
- 'COVERAGE_INFO': '/usr/obj/ports/Python-3.4.2/Python-3.4.2/coverage.info',
- 'COVERAGE_REPORT': '/usr/obj/ports/Python-3.4.2/Python-3.4.2/lcov-report',
+ 'COVERAGE_INFO': '/usr/obj/ports/Python-3.4.3/Python-3.4.3/coverage.info',
+ 'COVERAGE_REPORT': '/usr/obj/ports/Python-3.4.3/Python-3.4.3/lcov-report',
  'COVERAGE_REPORT_OPTIONS': '--no-branch-coverage --title "CPython lcov '
                             'report"',
  'CPPFLAGS': '-I. -IInclude -I./Include -I/usr/local/include',
@@ -183,6 +183,7 @@ build_time_vars = {'ABIFLAGS': 'm',
  'HAVE_GCC_UINT128_T': 1,
  'HAVE_GETADDRINFO': 1,
  'HAVE_GETC_UNLOCKED': 1,
+ 'HAVE_GETENTROPY': 1,
  'HAVE_GETGROUPLIST': 1,
  'HAVE_GETGROUPS': 1,
  'HAVE_GETHOSTBYNAME': 1,
@@ -296,6 +297,7 @@ build_time_vars = {'ABIFLAGS': 'm',
  'HAVE_PTY_H': 0,
  'HAVE_PUTENV': 1,
  'HAVE_PWRITE': 1,
+ 'HAVE_RAND_EGD': 0,
  'HAVE_READLINK': 1,
  'HAVE_READLINKAT': 1,
  'HAVE_READV': 1,
@@ -456,32 +458,32 @@ build_time_vars = {'ABIFLAGS': 'm',
  'HGBRANCH': '',
  'HGTAG': '',
  'HGVERSION': '',
- 'HOST_GNU_TYPE': 'mips64-unknown-openbsd5.7',
+ 'HOST_GNU_TYPE': 'sparc64-unknown-openbsd5.8',
  'INCLDIRSTOMAKE': '/usr/local/include /usr/local/include '
                    '/usr/local/include/python3.4m '
                    '/usr/local/include/python3.4m',
  'INCLUDEDIR': '/usr/local/include',
  'INCLUDEPY': '/usr/local/include/python3.4m',
- 'INSTALL': '/usr/obj/ports/Python-3.4.2/bin/install -c',
- 'INSTALL_DATA': '/usr/obj/ports/Python-3.4.2/bin/install -c  -m 644',
- 'INSTALL_PROGRAM': '/usr/obj/ports/Python-3.4.2/bin/install -c -s  -m 755',
- 'INSTALL_SCRIPT': '/usr/obj/ports/Python-3.4.2/bin/install -c  -m 755',
- 'INSTALL_SHARED': '/usr/obj/ports/Python-3.4.2/bin/install -c -m 555',
+ 'INSTALL': '/usr/obj/ports/Python-3.4.3/bin/install -c',
+ 'INSTALL_DATA': '/usr/obj/ports/Python-3.4.3/bin/install -c  -m 644',
+ 'INSTALL_PROGRAM': '/usr/obj/ports/Python-3.4.3/bin/install -c -s  -m 755',
+ 'INSTALL_SCRIPT': '/usr/obj/ports/Python-3.4.3/bin/install -c  -m 755',
+ 'INSTALL_SHARED': '/usr/obj/ports/Python-3.4.3/bin/install -c -m 555',
  'INSTSONAME': 'libpython3.4m.so.1.0',
  'IO_H': 'Modules/_io/_iomodule.h',
  'IO_OBJS': '\\',
  'LDCXXSHARED': 'c++ -pthread -shared -fPIC',
- 'LDFLAGS': '-L/usr/local/lib/ -L/usr/obj/ports/Python-3.4.2/Python-3.4.2',
+ 'LDFLAGS': '-L/usr/local/lib/ -L/usr/obj/ports/Python-3.4.3/Python-3.4.3',
  'LDLAST': '',
  'LDLIBRARY': 'libpython3.4m.so',
  'LDLIBRARYDIR': '',
  'LDSHARED': 'cc -pthread -shared -fPIC -L/usr/local/lib/ '
-             '-L/usr/obj/ports/Python-3.4.2/Python-3.4.2',
+             '-L/usr/obj/ports/Python-3.4.3/Python-3.4.3',
  'LDVERSION': '3.4m',
  'LIBC': '',
  'LIBDEST': '/usr/local/lib/python3.4',
  'LIBDIR': '/usr/local/lib',
- 'LIBFFI_INCLUDEDIR': '/usr/local/include',
+ 'LIBFFI_INCLUDEDIR': '',
  'LIBM': '-lm',
  'LIBOBJDIR': 'Python/',
  'LIBOBJS': '',
@@ -574,7 +576,7 @@ build_time_vars = {'ABIFLAGS': 'm',
  'PY_CPPFLAGS': '-I. -IInclude -I./Include -I/usr/local/include',
  'PY_FORMAT_LONG_LONG': '"ll"',
  'PY_FORMAT_SIZE_T': '"z"',
- 'PY_LDFLAGS': '-L/usr/local/lib/ -L/usr/obj/ports/Python-3.4.2/Python-3.4.2',
+ 'PY_LDFLAGS': '-L/usr/local/lib/ -L/usr/obj/ports/Python-3.4.3/Python-3.4.3',
  'Py_DEBUG': 0,
  'Py_ENABLE_SHARED': 1,
  'Py_HASH_ALGORITHM': 0,
@@ -583,7 +585,7 @@ build_time_vars = {'ABIFLAGS': 'm',
  'READELF': 'readelf',
  'RESSRCDIR': 'Mac/Resources/framework',
  'RETSIGTYPE': 'void',
- 'RUNSHARED': 'LD_LIBRARY_PATH=/usr/obj/ports/Python-3.4.2/Python-3.4.2',
+ 'RUNSHARED': 'LD_LIBRARY_PATH=/usr/obj/ports/Python-3.4.3/Python-3.4.3',
  'SCRIPTDIR': '/usr/local/lib',
  'SETPGRP_HAVE_ARG': 1,
  'SGI_ABI': '',
@@ -625,10 +627,10 @@ build_time_vars = {'ABIFLAGS': 'm',
  'TCLTK_LIBS': '',
  'TESTOPTS': '',
  'TESTPATH': '',
- 'TESTPYTHON': 'LD_LIBRARY_PATH=/usr/obj/ports/Python-3.4.2/Python-3.4.2 '
+ 'TESTPYTHON': 'LD_LIBRARY_PATH=/usr/obj/ports/Python-3.4.3/Python-3.4.3 '
                './python',
  'TESTPYTHONOPTS': '',
- 'TESTRUNNER': 'LD_LIBRARY_PATH=/usr/obj/ports/Python-3.4.2/Python-3.4.2 '
+ 'TESTRUNNER': 'LD_LIBRARY_PATH=/usr/obj/ports/Python-3.4.3/Python-3.4.3 '
                './python ./Tools/scripts/run_tests.py',
  'TESTTIMEOUT': 3600,
  'THREADOBJ': 'Python/thread.o',
@@ -653,8 +655,8 @@ build_time_vars = {'ABIFLAGS': 'm',
  'WITH_VALGRIND': 0,
  'X87_DOUBLE_ROUNDING': 0,
  'XMLLIBSUBDIRS': 'xml xml/dom xml/etree xml/parsers xml/sax',
- 'abs_builddir': '/usr/obj/ports/Python-3.4.2/Python-3.4.2',
- 'abs_srcdir': '/usr/obj/ports/Python-3.4.2/Python-3.4.2',
+ 'abs_builddir': '/usr/obj/ports/Python-3.4.3/Python-3.4.3',
+ 'abs_srcdir': '/usr/obj/ports/Python-3.4.3/Python-3.4.3',
  'datarootdir': '/usr/local/share',
  'exec_prefix': '/usr/local',
  'prefix': '/usr/local',
